unit MainU;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, cxGraphics, cxLookAndFeels, cxLookAndFeelPainters, Menus, StdCtrls,
  cxButtons, cxControls, cxContainer, cxEdit, cxGroupBox, cxRadioGroup,
  cxTextEdit, cxLabel, ExtCtrls, MAth, DB, ADODB, dxSkinsCore,
  dxSkinOffice2010Blue;

type
  TForm1 = class(TForm)
    cxGroupBox1: TcxGroupBox;
    ImgSQL_ConnectErr: TImage;
    ImgSQL_ConnectOK: TImage;
    lSQLDatabaseName: TcxLabel;
    edSQLDatabaseName: TcxTextEdit;
    lSQLServerServer: TcxLabel;
    lSQLConnecting: TcxLabel;
    edSQLServerName: TcxTextEdit;
    edSQLUserName: TcxTextEdit;
    lSQLUserName: TcxLabel;
    lSQLPassword: TcxLabel;
    edSQLPassword: TcxTextEdit;
    grpAuthentication: TcxRadioGroup;
    ADOConnection1: TADOConnection;
    cxLookAndFeelController1: TcxLookAndFeelController;
    procedure FormCreate(Sender: TObject);
  private
    FLogFileName: String;
    procedure SetupSqlServerFromCommandLine(const Value: string);
    function GetLogFileName: string;
    procedure AddToLog(Const Value: String);
  public
    procedure ValidateSQLProperties;
  end;

  TMsSQLAuthType = (auWindows, auSQLServer);

const
  sMsSQLAuthType: array [TMsSQLAuthType] of string = ('Windows Auth.', 'SQLServer Auth.');

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  CommandLineParserU;

function IfThen(AValue: Boolean; const ATrue: TMsSQLAuthType; AFalse: TMsSQLAuthType): TMsSQLAuthType; overload;
begin
  if AValue then
    Result := ATrue
  else
    Result := AFalse;
end;

procedure TForm1.AddToLog(const Value: String);
var
  LogPath: String;
  Buffer : TStringList;

begin
  LogPath := ExtractFilePath(FLogFileName);
  ForceDirectories(LogPath);
  Buffer :=  TStringList.Create;

  if FileExists(FLogFileName) then
    Buffer.LoadFromFile(FLogFileName);
  Buffer.Add(Value);
  Buffer.SaveToFile(FLogFileName);
  FreeAndNil(Buffer);
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  MSSQL: string;
begin
  ParameterArgChars(':=');
  FLogFileName := GetLogFileName;
  MSSQL := ParameterByName('MSSQL');
  SetupSqlServerFromCommandLine(MSSQL);
end;

function TForm1.GetLogFileName: string;
var
  LOG: string;
begin
  // Default:
  Result := ChangeFileExt(Application.ExeName, '_' + FormatDateTime('yyyymmdd_hhnnss', Now) + '.txt');
  Result := ChangeFilePath(Result, ExtractFileDir(Result) + '\Log\');

  LOG := ParameterByName('LOG');
  if LOG = '' then
    Exit;

  Result := LOG;

  // hvis ingen sti er angivet, s brug default
  if ExtractFilePath(Result) = '' then
    Result := ChangeFilePath(Result, ExtractFileDir(Application.ExeName) + '\Log\');
end;

procedure TForm1.SetupSqlServerFromCommandLine(const Value: string);
  function ParseSQLServerConStr(const AConStr: string; var AServerName, ADatabaseName, AUserName, APassword: string; var AMsSQLAuthType: TMsSQLAuthType): Boolean;
  var
    StringList: TStringList;
  begin
    AServerName := '';
    ADatabaseName := '';
    AUserName := '';
    APassword := '';
    AMsSQLAuthType := auSQLServer;
    Result := False;

    if Trim(AConStr) = '' then
      Exit;

    StringList := TStringList.Create;
    try
      StringList.LineBreak := ';';
      StringList.Text := AConStr;
      AServerName := StringList.Values['Data Source'];
      ADatabaseName := StringList.Values['Initial Catalog'];
      AUserName := StringList.Values['User ID'];
      APassword := StringList.Values['Password'];
      AMsSQLAuthType := IfThen(StringList.Values['Integrated Security'] = 'SSPI', auWindows, auSQLServer);
      Result := True;
    finally
      FreeAndNil(StringList);
    end;
  end;

var
  sServer, sDatabase, sUserName, sPassword: string;
  MsSQLAuthType: TMsSQLAuthType;
begin
  if Value = '' then
    Exit;

  AddToLog('SqlServer connectionstring baseret p commandline params: "' + Value + '"');

  ParseSQLServerConStr(Value, sServer, sDatabase, sUserName, sPassword, MsSQLAuthType);
  grpAuthentication.ItemIndex := IfThen(MsSQLAuthType = auWindows, 0, 1);
  edSQLServerName.Text := sServer;
  edSQLDatabaseName.Text := sDatabase;
  edSQLUserName.Text := sUserName;
  edSQLPassword.Text := sPassword;
  ValidateSQLProperties;
end;

procedure TForm1.ValidateSQLProperties;
var
  sDBName: string;
  MsSQLAuthType: TMsSQLAuthType;

  procedure ShowStatusIcons(const aConnected : Boolean);
  begin
    ImgSQL_ConnectErr.Visible := not aConnected;
    ImgSQL_ConnectOK.Visible := aConnected;
  end;

begin
  edSQLUserName.Text := Trim(edSQLUserName.Text);
  edSQLPassword.Text := Trim(edSQLPassword.Text);
  edSQLServerName.Text := Trim(edSQLServerName.Text);
  edSQLDatabaseName.Text := Trim(edSQLDatabaseName.Text);
  MsSQLAuthType := IfThen(grpAuthentication.ItemIndex = 0, TMsSQLAuthType.auWindows, TMsSQLAuthType.auSQLServer);
  sDBName := edSQLDatabaseName.Text;

  lSQLConnecting.Caption := 'Forsger at oprette forbindelse til SQL Server ...';
  lSQLConnecting.Visible := True;
  AddToLog(lSQLConnecting.Caption + ': ' + edSQLServerName.Text + '|' + sMsSQLAuthType[MsSQLAuthType] + '|' + edSQLUserName.Text + '|' + edSQLPassword.Text + '|' + sDBName);

  ImgSQL_ConnectErr.Visible := False;
  ImgSQL_ConnectOK.Visible := False;

  ADOConnection1.ConnectionString := ParameterByName('MSSQL');
  try
    ADOConnection1.Open;
    lSQLConnecting.Caption := 'Forbindelse til databasen opnet';
  except
    on e: Exception do
      lSQLConnecting.Caption := 'Forbindelse til databasen IKKE opnet, rsag: ' + e.Message;
  end;

  ShowStatusIcons(ADOConnection1.Connected);
  AddToLog(lSQLConnecting.Caption);
end;

end.
